
function [theta,epsilon_new,fenetre,accept_r,accept] = HMC_sample_v3(epsilon,L,theta_current,z,ii,n0,fenetre,varargin)
y=varargin{1};
psf=varargin{2};
sig2n=varargin{3};
xi=varargin{4};
scale=varargin{5};
K=varargin{6};
%% initialization
theta_cand=theta_current; 
m=1;
if isreal(theta_current)
    p=m*randn(size(theta_cand)); %the introduced momentum variables
else
    p=m*randn(size(theta_cand))+1j*m*randn(size(theta_cand));
end
current_p=p;
%% leapfrog method
% make a half step for momentum at the beginning
p=p-epsilon/2*grad_U(theta_cand,y,psf,sig2n,scale,xi,K,z);
% % alternate full steps for position and momentum
% p=-p;
for i=1:L
    %make a full step for the position
    theta_cand=theta_cand+epsilon*p/m;
    %make a full strp for the momentum, except at end of trajectory
    if (i~=L) 
       p=p-epsilon*grad_U(theta_cand,y,psf,sig2n,scale,xi,K,z); 
    end
end
%make a half step for momentum at the end
p=p-epsilon/2*grad_U(theta_cand,y,psf,sig2n,scale,xi,K,z);

%% evaluate potential and kinetic energies at start and end of trajectory
current_U=U(theta_current,y,psf,sig2n,scale,xi,K,z);
proposed_U=U(theta_cand,y,psf,sig2n,scale,xi,K,z);
current_K=norm(current_p(:))^2/(2*m);
proposed_K=norm(p(:))^2/(2*m); 
%% accept or reject the state at end od trajectory, 
%returning either the position at the end of the trajectory or the initial position
accept = min(exp(current_U-proposed_U+current_K-proposed_K),1);
% theta = theta*(rand<=accept)+theta_current*(rand>accept);
if rand<accept
     theta=theta_cand; 
     label=1;
else theta=theta_current; 
     label=0;
end


%% Adjust the stepsize adaptively
Nfenetre=30;
fenetre=[fenetre(1,2:Nfenetre),label];
accept_r=mean(fenetre);
    if ii-1<=n0
        if  accept_r<0.3
            epsilon_new=epsilon*(1-0.1);
        elseif accept_r>0.9
            epsilon_new= epsilon*(1+0.1);
        else
            epsilon_new=epsilon;
        end
    elseif ii-1>n0
        if  accept_r<0.3
        epsilon_new=epsilon*(1-0.1*exp(-0.01*(ii-n0)));
        elseif accept_r>0.9
        epsilon_new= epsilon*(1+0.1*exp(-0.01*(ii-n0)));
        else
            epsilon_new=epsilon;
        end 
    end
end

function U_energy=U(x,y,psf,sig2n,scale,xi,K,z)
%%%negetive log of the posterior function of x
% energy from the likelihood part
N=numel(x);
U_1= norm((y-Hx_cal(psf,x)),'fro')^2/(2*sig2n) + log(2*pi*sig2n)*N/2; 
% energy from the prior part
ind=cell(1,K);
x_sub=cell(1,K);
U_2=0;
for k=1:K
    ind{k}=find(z==k);
    x_sub{k}=x(ind{k});
    if isreal(x)
        U_2=U_2+(norm(x_sub{k}(:),xi(k))^xi(k))/scale(k)+numel(x_sub{k})*log(2*scale(k)^(1/xi(k))*gamma(1+1/xi(k)));
    else
        U_2=U_2+(norm(real(x_sub{k}),xi(k))^xi(k)+norm(imag(x_sub{k}),xi(k))^xi(k))/scale(k)+...
            2*numel(x_sub{k})*log(2*scale(k)^(1/xi(k))*gamma(1+1/xi(k)));
    end
end
U_energy=U_1+U_2;

end


function grad = grad_U(x,y,psf,sig2n,scale,xi,K,z)
 
% gradient from the likelihood part
grad1=HTx_cal(psf,Hx_cal(psf,x)-y)/sig2n;
% grad1=KTx_lin(Kx_lin(x,psf)-y,psf)/sig2n;

% gradient from the prior part
ind=cell(1,K);
x_sub=cell(1,K);
grad2=cell(1,K);
grad_p=zeros(size(grad1));
c=0.0001;
for k=1:K
    ind{k}=find(z==k);
    x_sub{k}=x(ind{k});
    d{k}=sqrt(abs(real(x_sub{k})).^2+c)+j*sqrt(abs(imag(x_sub{k})).^2+c);
    if isreal(x)
        grad2{k}=xi(k)*x_sub{k}.*(abs(x_sub{k}).^(xi(k)-2))/scale(k); 
    else
        grad2{k}=xi(k)*real(d{k}).*(abs(real(d{k})).^(xi(k)-2))/scale(k)+...
            xi(k)*imag(d{k}).*(abs(imag(d{k})).^(xi(k)-2))/scale(k);
    end
%     d{k}=sqrt(abs(x_sub{k}).^2+c);
%     grad2{k}=xi(k)*d{k}.*x_sub{k}.*(d{k}.^(xi(k)-2))/scale(k);
    grad_p(ind{k})=grad2{k};
end
grad=grad1+grad_p;

end
