function [parms] = ggd_fit(data,varargin)
%
% nu : the shape paramter
% s  : the variance parameter
% scale : the scale paramter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin==3
    nu = varargin{1};
    scale =varargin{2};
else return;
end

data = data(:);
n = numel(data);

xmin = quantile(data,0.01);
xmax = quantile(data,0.99);
xmax = max([abs(xmin) xmax]);
xmin = -xmax;

NbBin = round(sqrt(n));
step = 2*xmax/NbBin;
edges = xmin:step:xmax;

x = [-xmax:.01:xmax];

%calculate the empirical pdf
histo = hist(data(data<=xmax),edges);
histo = histo /(step*sum(histo));
histo(end)=histo(end-1);

[Ni] = histc(data,edges);
Ni = Ni/trapz(edges,Ni); %normalization of the histogram

% ggd fit
x = [-xmax:.01:xmax];

pdf = nu/(2*scale^(1/nu)*gamma(1/nu))*exp(-abs(x.^nu)./scale);

figure,
    b = plot(edges(1:end), histo(1:end),'Color',[0.7 0.7 0.7],'Linewidth',2);
    hold on
    g = plot(x,pdf,'r','linewidth',2);
    legend([b g],{'amplitude histogram',...
        ['GGD(\xi=' num2str(nu) ',scale=' num2str(scale) ')'] })
    axis tight
    xlim([-xmax xmax])


parms = [nu, scale]; % shape and variance

end


