function [h] = hist_fit(data,xi,s,varargin)
%histogram fit of generalized Gaussian distribution
%input: xi-shape parameter
%input: s-scale 
%input: data, data must be a vector

if nargin<3
   error('Not Enough Inputs'); 
elseif nargin==5
    xi_ref=varargin{1};
    s_ref=varargin{2};
end

if isvector(data)
    
n = numel(data);
xmin = quantile(data,0.01);
xmax = quantile(data,0.99);
xmax = max([abs(xmin) xmax]);
xmin = -xmax;

NbBin = round(sqrt(n));
step = 2*xmax/NbBin;
edges = xmin:step:xmax;

x = [-xmax:.01:xmax];

%calculate the empirical pdf
histo = hist(data(data<=xmax),edges);
histo = histo /(step*sum(histo));
histo(end)=histo(end-1);

% if numel(xi)==1 && numel(s)==1
    pdf = xi/(2*s^(1/xi)*gamma(1/xi))*exp(-abs(x).^xi./s);
    if nargin==5
    pdf_ref = xi_ref/(2*s_ref^(1/xi_ref)*gamma(1/xi_ref))*exp(-abs(x).^xi_ref./s_ref);
    end
%     figure;
    plot(edges(1:end), histo(1:end),'Color',[0.7 0.7 0.7],'Linewidth',2)
    
    hold on
    g = plot(x,pdf,'b--','linewidth',2); hold off
        hleg1 = legend('Truth',...
        ['GGD(\xi=' num2str(xi) ',\gamma=' num2str(s) ')'] );
    if nargin==5
        hold on,
    g1 = plot(x,pdf_ref,'r--','linewidth',2); hold off
        hleg1 = legend('Truth',...
        ['GGD(\xi=' num2str(xi) ',\gamma=' num2str(s) ')'], ['GGD(\xi=' num2str(xi_ref) ',\gamma=' num2str(s_ref) ')'] );
    end

 %set(hleg1,'Interpreter','latex')
 set(gca,'fontsize',15)
    axis tight
    xlim([-xmax xmax])
    
% elseif size(data,1)==2
% vara=data;
% for i=1:2
%     data=vara(i,:);
% n = numel(data);
% xmin = quantile(data,0.01);
% xmax = quantile(data,0.99);
% xmax = max([abs(xmin) xmax]);
% xmin = -xmax;
% 
% NbBin = round(sqrt(n));
% step = 2*xmax/NbBin;
% edges = xmin:step:xmax;
% 
% x = [-xmax:.01:xmax];
% 
% %calculate the empirical pdf
% histo = hist(data(data<=xmax),edges);
% histo = histo /(step*sum(histo));
% histo(end)=histo(end-1);
% histogram(i,:)=histo;
% end
% pdf = xi/(2*s^(1/xi)*gamma(1/xi))*exp(-abs(x).^xi./s);
% 
% % figure;
% plot(edges(1:end), histogram(1,:),'Color',[0.2 0.2 0.2],'Linewidth',2)
% hold on
% plot(edges(1:end), histogram(2,:),'Color',[0.7 0.7 0.7],'Linewidth',2)
% hold on
% g = plot(x,pdf,'b--','linewidth',2);
% hleg1 = legend('Empirical','Estimation',...
%     ['GGD(\xi=' num2str(xi) ',\gamma=' num2str(s) ')'] );
% %set(hleg1,'Interpreter','latex')
% set(gca,'fontsize',15)
% axis tight
% xlim([-xmax xmax])
% end
end 
