function [z_new,ind] = label_sample_MRF_4neighbours( z_old,x,K,beta,xi,scale,mu,ch)
%--------------------------------------------------------------------------
%DATE: 2014/06/06
%UPDATE: 2014/06/20
%AUTHOR: N. ZHAO (IRIT/ENSEEIHT)
%
%this code is to use Potts Markov fields as the prior of the labels
%the number of neighbour element is 4
%using checkeboard, half of the labels can be sampled at one time 
%
%INPUT ch    : if ch==1 pixels in blackboard will be considered
%              if ch==2 pixels in whiteboard will be considered
%      z_old : labels at previous iteration (matrix)
%         xi : shape
%       scale: scale
%       beta : granularity
%OUTPUT z_new: labels at current ieration (vector)
%        ind : index of z_new in the matrix
%--------------------------------------------------------------------------

[m,n]=size(z_old);
[b{1},b{2}]=checkerboard(m,n);
pi=zeros(m,n,K);
board=zeros(m,n,K);
posterior_pi=zeros(m,n,K);

    
if ch==1
    ind=find(b{1}==1);
elseif ch==2
    ind=find(b{2}==1);
end

for k=1:K
    board(:,:,k)=neighbours(z_old,k,ch,b{1},b{2});
%     pi(:,:,k)=exp(beta*board(:,:,k));%x \sim constant
%     pi(:,:,k)=exp(beta*board(:,:,k)).*exp(-(abs(x).^2)/scale(k))/sqrt(scale(k));%gaussian (likelihood) case
    if isreal(x)
        pi(:,:,k)=exp(beta*board(:,:,k)).*exp(-abs(x-mu(k)).^xi(k)/scale(k))/(2*scale(k)^(1/xi(k))*gamma(1+1/xi(k)));%GGD (likelihood)
    else
        pi(:,:,k)=exp(beta*board(:,:,k)).*exp(-(abs(real(x)-mu(k)).^xi(k)+abs(imag(x)-mu(k)).^xi(k))/scale(k))/(2*scale(k)^(1/xi(k))*gamma(1+1/xi(k)));%GGD (likelihood)
    end
%     pi(:,:,k)=exp(beta*board(:,:,k)).*x.^(xi(k)-1).*exp(-x/scale(k))/(gamma(xi(k))*scale(k)^xi(k));%Gamma (likelihood) case
end
sum_pi=cumsum(pi,3);
sum_pi=sum_pi(:,:,K);
for k=1:K
    posterior_pi(:,:,k)=pi(:,:,k)./sum_pi.*b{ch};
end
posterior_pi=cumsum(posterior_pi,3);
temp=z_old(ind);
bound=zeros(numel(temp),K+1);
rho=rand(numel(temp),1);

bound(:,1)=zeros(size(temp));
temp=(bound(:,1)<=rho);
for i=2:K+1
    tt=posterior_pi(:,:,i-1);
    bound(:,i)=tt(ind);
    temp=temp+(bound(:,i)<=rho);
end
z_new=temp;

end

function [b1,b2]=checkerboard(m,n)
N=max(m,n);
pos1          = ones(1,N);
pos1(2:2:end) = 0;
pos1          = toeplitz([pos1]);
pos1=pos1(1:m,1:n);
b1=pos1;
b2=1-b1;
end

function [board]=neighbours(z,k,ch,b1,b2)

neighbour1 = circshift(z, [0 -1]);
neighbour2 = circshift(z, [0 1]);
neighbour3 = circshift(z, [-1 0]);
neighbour4 = circshift(z, [1 0]);
board=((k-neighbour1)==0) + ((k-neighbour2)==0) + ((k-neighbour3)==0) + ((k-neighbour4)==0);
if ch==1
    board=board.*b1;
elseif ch==2
    board=board.*b2;
end
end





        
    


